@@ -819,6 +819,7 @@ if ($cfg->{checksum}) {
 
     foreach (glob('*{,/*}')) {
 	next if -d; # Skip directories
+	(my $vkey = $_) =~ s/%5f/_/g; # Decode embedded underscores
 	if (/([-+.a-z0-9]+_(?:\d:)?[-+.~a-zA-Z0-9]+)_changelog$/ && !$svrhash{$1}) { # Changelogs
 	    unlink $_, "../headers/$_" unless $sim_mode;
 	    printmsg "Removing expired changelog: $_ and company...\n";
@@ -830,23 +831,23 @@ if ($cfg->{checksum}) {
 	}
 	elsif (is_file_type('package', $_) or is_file_type('pdiff', get_original_url($_))) { # Package and pdiff files
 
-	    if (! defined($valid{$_})) {
+	    if (! defined($valid{$vkey})) {
 		unlink $_, "../headers/$_", "../private/$_.complete" unless $sim_mode;
 		printmsg "Removing file: $_ and company...\n";
 	    } else {
-		# Verify SHA1 checksum
-		my $target_sum = hashify(\$valid{$_})->{sha1};
+		# Verify SHA256 checksum
+		my $target_sum = hashify(\$valid{$vkey})->{sha256};
 		next unless $target_sum;
-		# print "Validating SHA1 $target_sum for $_\n";
+		# print "Validating SHA256 $target_sum for $_\n";
 		open(my $fh, '<', $_) || die "Unable to open file $_ to verify checksum: $!";
 		flock($fh, LOCK_EX);
 		if (is_file_type('pdiff', get_original_url($_))) { # pdiffs need decompressing
 		    $fh = IO::Uncompress::AnyUncompress->new($fh)
 		      or die "Decompression failed: $AnyUncompressError\n";
 		}
-		if ((my $sha1 = Digest::SHA->new(1)->addfile($fh)->hexdigest) ne $target_sum) {
+		if ((my $sha256 = Digest::SHA->new(256)->addfile($fh)->hexdigest) ne $target_sum) {
 		    unlink $_, "../headers/$_", "../private/$_.complete" unless $sim_mode;
-		    printmsg "Checksum mismatch ($target_sum <=> $sha1): $_, removing\n";
+		    printmsg "Checksum mismatch ($target_sum <=> $sha256): $_, removing\n";
 		}
 		# No explicit LOCK_UN: it fails with IO::Uncompress::AnyUncompress, just rely on close
 		close $fh;
@@ -861,7 +862,8 @@ chdir "$cfg->{cache_dir}/headers" && -w "." || die "Could not enter the cache he
 
 foreach (glob('*{,/*}')) {
     next if -d; # Skip directories
-    if((is_file_type('package', $_) && !defined($valid{$_})) # Not indexed
+    (my $vkey = $_) =~ s/%5f/_/g; # Decode embedded underscores
+    if((is_file_type('package', $_) && !defined($valid{$vkey})) # Not indexed
        || !-e "../packages/$_") { # No corresponding package
 	unlink $_, "../private/$_.complete" unless $sim_mode;
 	printmsg "Removing expired headers: $_ and company...\n";
@@ -870,7 +872,7 @@ foreach (glob('*{,/*}')) {
     my $resp = read_header($_);
     next if $resp && $resp->is_success; # Don't cache errors any more
     printmsg "Removing cached error/invalid response: $_\n";
-    delete $valid{$_};
+    delete $valid{$vkey};
     unlink $_ unless $sim_mode;
 }
 
